# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.
# See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Thu Jan 19 16:09:01 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:aa0026/cmdb.git $
#

from pkg_resources import get_distribution, DistributionNotFound

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __package__
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:  # pragma: no cover
    # Can't unit test this for now since I do not know how to temporarily
    # remove a distribution during runtime. Help. Also, is this even
    # worth it?
    __version__ = "unknown"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.
# See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Thu Feb 2 13:43:56 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECScmdb.git $
#

import sys
import logging
import logging.handlers

from datetime import datetime
from socket import gethostname
from pathlib import Path
import autologging
import pandas as pd

from autologging import logged, traced
from docopt import docopt
from jinja2 import Template

import ecspylibs
from ecspylibs.reapchildren import ReapChildren
from ecspylibs.sendemail import SendEmail

import ecscmdb
from ecscmdb.initcheck import InitCheck
from ecscmdb.finddifferences import FindDifferences

global df_summary


@traced
@logged
def main():
    """
Program to analyze two spreadsheets for differences.

Some default option values listed below can be overridden within the initialization file.

Usage:
  %(pgm)s [-v] [-L LEVEL] [--LOG=DIR] [-c CONFIG] [-s SECTION] [-r REPORT] [-D] SPREADSHEET1 SPREADSHEET2
  %(pgm)s (-h | --help | -V | --version)

  Variables SPREADSHEET1 and SPREADSHEET2 are required, all other parameters are optional.

Options:
  -h, --help                          Show this help message and exit.
  -V, --version                       Show version information and exit.
  -c CONFIG, --config=CONFIG          The configuration file.
                                      Default: "%(--config)s"
  -s SECTION, --section=SECTION       The configuration file version (default
                                      defined within the configuration file).
  -r REPORT, --report=REPORT          Report directory or file.
  -v, --verbose                       Print verbose messages.
  -L LEVEL, --log=LEVEL               Print log messages at log value LEVEL.
                                      Valid levels are: TRACE, DEBUG, INFO, WARNING,
                                      ERROR, and CRITICAL.
                                      Default: "%(--log)s"
  --LOG=DIR                           Log Directory,
                                      Default: "%(--LOG)s"
  -D, --dryrun                        Only print out what would be done.
"""

    @traced
    @logged
    def get_col_widths(df):
        """Blah blah blah."""

        return [int(max([len(str(s)) for s in df[col].values] + [len(col)]) + 2.0) for col in df.columns]

    @traced
    @logged
    def fix_col_widths(ws, df):
        """Blah blah blah."""

        for i, width in enumerate(get_col_widths(df)):
            ws.set_column(i, i, width)

    TOD = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
    DATE = datetime.now().strftime("%A, %B %e, %Y")
    TIME = datetime.now().strftime("%r")

    FQ_RUN_DIR = Path.cwd().resolve(strict=True)
    FQ_PGM_FILE = Path(__file__).resolve(strict=True)
    assert len(FQ_PGM_FILE.parts) >= 2
    PGM = Path(FQ_PGM_FILE.stem)
    FQ_BASE_DIR = FQ_PGM_FILE.parents[2]
    PKG = Path(__package__)

    ETC_DIR = Path("etc")
    REPORT_DIR = Path("report")
    LOG_DIR = Path("log")

    CONF_FILE = PGM.with_suffix(".yml")

    REPORT_NAME = Path(f"OpenManage-{PGM}.{TOD}.xlsx")
    LOG_FILE = PGM.with_suffix(".log")

    CONF_LIST = [
        FQ_RUN_DIR / ETC_DIR / PKG / CONF_FILE,
        FQ_RUN_DIR / ETC_DIR / CONF_FILE,
        FQ_RUN_DIR / CONF_FILE,
        FQ_BASE_DIR / ETC_DIR / PKG / CONF_FILE,
        FQ_BASE_DIR / ETC_DIR / CONF_FILE,
        FQ_BASE_DIR / CONF_FILE
    ]

    for tmp in CONF_LIST:
        if tmp.is_file():
            FQ_CONF_FILE = tmp.resolve(strict=True)
            break
    else:
        print(f"Couldn't find Configuration File '{CONF_FILE.name}'.")
        sys.exit(1)

    FQ_REPORT_DIR = (FQ_RUN_DIR / REPORT_DIR).resolve(strict=False)
    FQ_REPORT_FILE = (FQ_REPORT_DIR / REPORT_NAME).resolve(strict=False)

    FQ_LOG_DIR = (FQ_RUN_DIR / LOG_DIR).resolve(strict=False)
    FQ_LOG_FILE = (FQ_LOG_DIR / LOG_FILE).resolve(strict=False)

    # Get hostname.

    HOST = gethostname()

    help_arguments = {
        "pgm": PGM,
        "--config": FQ_CONF_FILE,
        "--LOG": FQ_LOG_FILE,
        "--log": "WARNING",
        "--report": FQ_REPORT_FILE,
    }

    help_text = main.__doc__ % help_arguments

    pgm_version = f"{PGM}: {__package__}({ecscmdb.__version__}), ecspylibs({ecspylibs.__version__})"

    arguments = docopt(help_text, version=pgm_version)

    CONFIG = InitCheck(arguments, help_arguments, config="--config", section="--section")

    arguments = CONFIG.arguments
    data = CONFIG.data

    dry_run = arguments["--dryrun"]
    log_level = arguments["--log"].upper()
    log_fd = arguments["--LOG"]
    report = arguments["--report"]
    verbose = arguments["--verbose"]
    version = arguments["--version"]

    report_file = CONFIG.report_file
    log_file = CONFIG.log_file

    # Setup logging.

    log_levels = {
        "CRITICAL": logging.CRITICAL,
        "DEBUG": logging.DEBUG,
        "ERROR": logging.ERROR,
        "INFO": logging.INFO,
        "TRACE": autologging.TRACE,
        "WARNING": logging.WARNING,
    }

    logger = logging.getLogger()
    logger.setLevel(log_levels[log_level])
    log_file = log_fd

    fh = logging.handlers.RotatingFileHandler(log_file, maxBytes=(50 * 1024 ** 2), backupCount=5)

    log_fmt = ""
    log_fmt += "%(asctime)s"
    log_fmt += " %(levelname)-7s"
    log_fmt += " [%(process)5d]"
    log_fmt += " %(module)s.py(%(funcName)s,%(lineno)d):"
    log_fmt += "\t%(message)s"

    formatter = logging.Formatter(log_fmt)

    fh.setFormatter(formatter)

    logger.addHandler(fh)

    logger.info(f"Starting program {pgm_version}.")
    if arguments["--verbose"]:
        print(f"Starting program {pgm_version}.", file=sys.stdout)

    if dry_run:
        print(f"\nList of command line arguments.\n", file=sys.stdout)

    for key in sorted(arguments.keys()):
        logger.debug(f"arguments['{key}']\t= '{arguments[key]}'")
        if dry_run:
            print(f"arguments['{key}']\t= '{arguments[key]}'", file=sys.stdout)

    del key

    my_locals = locals()
    my_locals_keys = my_locals.keys()
    my_locals_vals = my_locals.values()
    del my_locals

    if dry_run:
        print(f"\nList of all variables.\n", file=sys.stdout)

    for k, v in zip(my_locals_keys, my_locals_vals):
        logger.debug(f"{k:20} = '{v}'")
        if dry_run:
            print(f"{k:20} = '{v}'", file=sys.stdout)

    if dry_run:
        print("", file=sys.stdout)
        sys.exit(0)

    email_from = CONFIG.email_from
    email_to = CONFIG.email_to
    email_cc = CONFIG.email_cc
    subject = CONFIG.email_subject
    email_text_with_changes = CONFIG.email_text_with_changes
    email_text_without_changes = CONFIG.email_text_without_changes
    email_text_with_changes = CONFIG.email_text_with_changes
    email_text_without_changes = CONFIG.email_text_without_changes

    admin_name = CONFIG.admin_name
    admin_email = CONFIG.admin_email
    admin_phone = CONFIG.admin_phone

    spread_sheet1 = Path(arguments["SPREADSHEET1"]).resolve(strict=False)
    spread_sheet2 = Path(arguments["SPREADSHEET2"]).resolve(strict=False)

    try:
        spread_sheet1 = spread_sheet1.resolve(strict=True)
    except FileNotFoundError as e:
        logger.error(f"SPREADSHEET1 is missing or invalid: '{spread_sheet1}'")
        print(f"\nError: SPREADSHEET1 is missing or invalid: '{spread_sheet1}'\n", file=sys.stderr)
    except Exception as e:
        logger.error(f"\nException: {e=}")
        print(f"\nException: {e=}\n", file=sys.stderr)
        sys.exit(1)

    try:
        spread_sheet2 = spread_sheet2.resolve(strict=True)
    except FileNotFoundError as e:
        logger.error(f"SPREADSHEET2 is missing or invalid: '{spread_sheet2}'")
        print(f"\nError: SPREADSHEET2 is missing or invalid: '{spread_sheet2}'\n", file=sys.stderr)
    except Exception as e:
        print(f"\nException: {e=}\n", file=sys.stderr)
        sys.exit(1)

    short_spread_sheet1 = spread_sheet1.name
    short_spread_sheet2 = spread_sheet2.name

    if dry_run:
        print(f"\nList of command line arguments.\n", file=sys.stdout)

    for key in sorted(arguments.keys()):
        logger.debug(f"arguments['{key}']\t= '{arguments[key]}'")
        if dry_run:
            print(f"arguments['{key}']\t= '{arguments[key]}'", file=sys.stdout)

    del key

    my_locals = locals()
    my_locals_keys = my_locals.keys()
    my_locals_vals = my_locals.values()
    del my_locals

    if dry_run:
        print(f"\nList of all variables.\n", file=sys.stdout)

    for k, v in zip(my_locals_keys, my_locals_vals):
        logger.debug(f"{k:20} = '{v}'")
        if dry_run:
            print(f"{k:20} = '{v}'", file=sys.stdout)

    if dry_run:
        print("", file=sys.stdout)
        sys.exit(0)

    # The real stuff goes here.

    excel = pd.ExcelWriter(report)

    logger.debug(f"Create df_spread_sheet1 from file '{spread_sheet1}'.")
    try:
        df_spread_sheet1 = pd.read_excel(spread_sheet1, sheet_name=None)
    except Exception as e:
        logger.critical(f"Failed to read '{spread_sheet1}' Spreadsheet.")
        logger.exception(f"Exception: '{e}'.")
        logger.info(f"Ending program {pgm_version}.")
        if verbose:
            print(f"\nEnding program {pgm_version}.\n", file=sys.stdout)
        sys.exit(1)

    logger.debug(f"Create df_spread_sheet2 from file '{spread_sheet2}'.")
    try:
        df_spread_sheet2 = pd.read_excel(spread_sheet2, sheet_name=None)
    except Exception as e:
        logger.critical(f"Failed to read '{spread_sheet2}' spreadsheet.")
        logger.exception(f"Exception: '{e}'.")
        logger.info(f"Ending program {pgm_version}..")
        if verbose:
            print(f"\nEnding program {pgm_version}.\n", file=sys.stdout)
        sys.exit(1)

    sheet_names1 = set(df_spread_sheet1.keys())
    logger.debug(f"sheet_names1: '{sheet_names1}'")
    worksheets1 = len(sheet_names1)
    logger.debug(f"worksheets1: '{worksheets1}'")

    sheet_names2 = set(df_spread_sheet2.keys())
    logger.debug(f"sheet_names2: '{sheet_names2}'")
    worksheets2 = len(sheet_names2)
    logger.debug(f"worksheets2: '{worksheets2}'")

    deleted_sheet_names1 = sheet_names1.difference(sheet_names2)
    logger.debug(f"deleted_sheet_names1: '{deleted_sheet_names1}'")

    deleted_sheet_names2 = sheet_names2.difference(sheet_names1)
    logger.debug(f"deleted_sheet_names2: '{deleted_sheet_names2}'")

    common_sheet_names = list(sheet_names1.intersection(sheet_names2))
    common_sheet_names.sort()
    logger.debug(f"common_sheet_names: '{common_sheet_names}'")

    if verbose:
        print(f"\n=====================================================\n", file=sys.stdout)
        print(f"Analyzing spreadsheet:\n\t '{short_spread_sheet1}'\n", file=sys.stdout)
        print(f"with spreadsheet:\n\t '{short_spread_sheet2}'", file=sys.stdout)
        print(f"\n=====================================================\n", file=sys.stdout)

    logger.info(f"Creating the Summary sheet.")

    # "Summary of changes." = f"Summary of changes."
    details_column = f"See sheet tab for details."
    df_summary = pd.DataFrame()
    df_summary["Summary of changes."] = ""
    df_summary["See sheet tab for details."] = ""

    cols = df_summary.columns.tolist()
    cols.insert(0, cols.pop(cols.index("Summary of changes.")))

    df_summary = df_summary.reindex(columns=cols)
    df_summary.to_excel(excel, "Summary", index=False)

    df_summary = df_summary.append(
        {
            "Summary of changes.": f"Comparing '{short_spread_sheet1}' with '{short_spread_sheet2}' on {DATE} {TIME}.",
        }, ignore_index=True)

    df_summary = df_summary.append(
        {
            "Summary of changes.": f"All changes listed in this Summary page are relative to '{short_spread_sheet2}'.",
        }, ignore_index=True)

    df_summary = df_summary.append(
        {
            "Summary of changes.": f"",
        }, ignore_index=True)

    data_changes = False

    if verbose:
        print(f"Looking for added or deleted sheet names in spreadsheets:", end="", file=sys.stdout)

    if deleted_sheet_names1 or deleted_sheet_names2:
        data_changes = True

        if verbose:
            print(f"\n\nThe following sheet name(s) changed in the spreadsheet:\n", file=sys.stdout)

        if deleted_sheet_names2:
            if verbose:
                print(f"Sheet name(s) added:", file=sys.stdout)

            for added in deleted_sheet_names2:
                if verbose:
                    print(f"\t{added}", file=sys.stdout)

                df_summary = df_summary.append(
                    {
                        "Summary of changes.": f"Sheet '{added}' added.",
                        "See sheet tab for details.": f"{added}",
                    }, ignore_index=True)

                df_sheet = pd.DataFrame(df_spread_sheet2[added])
                df_sheet.to_excel(excel, added, index=False)
                fix_col_widths(excel.sheets[added], df_sheet)

        if deleted_sheet_names1 and deleted_sheet_names2:
            if verbose:
                print(f"", file=sys.stdout)

        if deleted_sheet_names1:
            if verbose:
                print(f"Sheet name(s) deleted:", file=sys.stdout)

            for deleted in deleted_sheet_names1:
                if verbose:
                    print(f"\t{deleted}", file=sys.stdout)

                df_summary = df_summary.append(
                    {
                        "Summary of changes.": f"Sheet '{deleted}' deleted.",
                        "See sheet tab for details.": f"{deleted}",
                    }, ignore_index=True)

                df_sheet = pd.DataFrame(df_spread_sheet1[deleted])
                df_sheet.to_excel(excel, deleted, index=False)
                fix_col_widths(excel.sheets[deleted], df_sheet)

        if verbose:
            print(f"\nThe sheet name(s) listed above will be ignored when analyzing the spreadsheets.\n",
                  file=sys.stdout)

    else:
        logger.info(f"There were no sheets added or deleted in either of the CMDB spreadsheets.")
        if verbose:
            print(f" None", file=sys.stdout)

        df_summary = df_summary.append(
            {
                "Summary of changes.": f"There were no sheets added or deleted in either of the CMDB spreadsheets.",
                "See sheet tab for details.": f"None",
            }, ignore_index=True)

    data_changes = False

    print(f"Looking for sheets with data changes.", file=sys.stdout)

    find_diffs = FindDifferences(df_spread_sheet1, df_spread_sheet2, df_summary)

    for sheet in common_sheet_names:
        df_spread_sheet1[sheet] = df_spread_sheet1[sheet].fillna("")
        df_spread_sheet2[sheet] = df_spread_sheet2[sheet].fillna("")

        if not df_spread_sheet2[sheet].equals(df_spread_sheet1[sheet]):
            logger.info(f"Sheet '{sheet}' has changes.")
            print(f"Sheet '{sheet}' has changes.", file=sys.stdout)

            df_style, df_sheet, df_summary = find_diffs.find_differences(sheet)
            df_style = df_sheet.style.apply(find_diffs.set_style, style=df_style, axis=None)

            data_changes = True

            df_style.to_excel(excel, sheet, index=False)
            fix_col_widths(excel.sheets[sheet], df_sheet)

        else:
            logger.info(f"Sheet '{sheet}' has no changes.")

    if not data_changes:
        logger.info(f"There was no data changes in any sheet in either of the CMDB spreadsheets.")
        if verbose:
            print(f"There was no data changes in any sheet in either of the CMDB spreadsheets.", file=sys.stdout)

        df_summary = df_summary.append(
            {
                "Summary of changes.": f"There was no data changes in any sheet in either of the CMDB spreadsheets.",
                "See sheet tab for details.": f"None",
            }, ignore_index=True)

        print(f"\n=====================================================\n", file=sys.stdout)

    logger.info(f"Update the Summary sheet.")
    df_summary.to_excel(excel, "Summary", index=False)

    logger.info(f"Fix Column Widths for the Summary sheet.")
    fix_col_widths(excel.sheets["Summary"], df_summary)

    logger.info(f"Write out the Report spreadsheet.")
    excel.save()
    excel.close()

    logger.info(f"Create report.")

    # Fix all jinja2 strings.

    replacement = {
        "ADMINEMAIL": admin_email,
        "ADMINNAME": admin_name,
        "ADMINPHONE": admin_phone,
        "DATE": DATE,
        "HOST": HOST,
        "REPORT": report_file.name,
        "SPREADSHEET1": short_spread_sheet1,
        "SPREADSHEET2": short_spread_sheet2,
        "TIME": TIME,
        "VERSION": pgm_version,
        "WORKSHEETS1": worksheets1,
        "WORKSHEETS2": worksheets2,
    }

    if data_changes:
        logger.info(f"Using text for when data changed between spreadsheets.")
        text = email_text_with_changes
    else:
        logger.info(f"Using text for when data did not change between spreadsheets.")
        text = email_text_without_changes

    logger.debug(f"text(1) : {text}")

    logger.info(f"Updating variables email_from, subject, and text.")
    email_from = Template(email_from).render(replacement)
    subject = Template(subject).render(replacement)
    text = Template(text).render(replacement)

    logger.debug(f"email_from : '{email_from}'.")
    logger.debug(f"email_to : '{email_to}'.")
    logger.debug(f"email_cc : '{email_cc}'.")
    logger.debug(f"subject : '{subject}'.")
    logger.debug(f"text(2) : '{text}'")

    sm = SendEmail()
    sm.send_email(email_from, email_to, email_cc, subject, text, text_type="plain",
                  files=[spread_sheet1, spread_sheet2, report], server="localhost")

    rc = ReapChildren()
    try:
        rc.reap_children()
    except Exception as e:
        logger.critical(f"Fatal error occurred in reapChildren().")
        logger.exception(f"Exception: '{e}'.")

    logger.info(f"Ending program {pgm_version}.")
    if verbose:
        print(f"\nEnding program {pgm_version}.\n", file=sys.stdout)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.
# See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Wed Feb 1 13:59:53 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECScmdb.git $
#

import json
import sys
import pandas as pd
import requests

from ecspylibs.password import Password
from autologging import logged, traced
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


@traced
@logged
class API:
    """Verify blah blah blah."""

    def __init__(self, user, passwd_file):
        """Set up the CheckService environment."""

        self.inventory_types_url = None
        self.inventory_details_url = None
        self.device_id = None
        self.device_response = None
        self.session_info = None
        self.ome_user = user
        self.passwd_file = passwd_file

        self.owm_passwd_db = Password(self.passwd_file)

        self.ome_passwd = self.owm_passwd_db.get(self.ome_user)
        self.ome_host = self.owm_passwd_db.get("OMEHost")

        del self.owm_passwd_db

        self.headers = {
            "content-type": "application/json",
        }

        self.user_creds = {
            "UserName": self.ome_user,
            "Password": self.ome_passwd,
            "SessionType": "API",
        }

        self.__log.debug(f"User            : '{self.ome_user}'")
        self.__log.debug(f"Pass            : '{self.ome_passwd}'")
        self.__log.debug(f"Host            : '{self.ome_host}'")
        self.__log.debug(f"OMESessionURL   : 'https://{self.ome_host}/api/SessionService/Sessions'")
        self.__log.debug(f"OMEDeviceURL    : 'https://{self.ome_host}/api/DeviceService/Devices'")

    def login(self):
        """ Blah."""

        try:
            self.session_info = requests.post(f"https://{self.ome_host}/api/SessionService/Sessions",
                                              verify=False,
                                              data=json.dumps(self.user_creds), headers=self.headers)
            self.__log.debug(f"sessionInfo.status_code = '{self.session_info.status_code}'")
            self.__log.debug(f"sessionInfo.headers")
            for self.i in self.headers.keys():
                self.__log.debug(f"\tsessionInfo.headers[{self.i}] = '{self.headers[self.i]}'")
        except Exception as e:
            print(f"Exception: '{e}'.", file=sys.stderr)
            self.__log.critical(f"Fatal error occurred while something.")
            self.__log.exception(f"Exception: '{e}'.")
            return False

        self.__log.debug(f"sessionInfo.status_code = '{self.session_info.status_code}'")
        if self.session_info.status_code == 201:
            self.headers["X-Auth-Token"] = self.session_info.headers["X-Auth-Token"]
        else:
            print(f"Unable to create a session with appliance {self.ome_host}", file=sys.stderr)
            self.__log.error(f"Unable to create a session with appliance {self.ome_host}")
            return False

        return True

    def _get_device_list(self, next_link=None):
        """Verify blah blah blah

    Return codes:
        0 :
        1 :
        2 :
        3 :
        4 :
    """

        if next_link:
            self.__log.debug(f"nextLink : '{next_link}'")
            self.URL = f"https://{self.ome_host}{next_link}"
        else:
            self.__log.debug(f"nextLink : None")
            self.URL = f"https://{self.ome_host}/api/DeviceService/Devices"

        self.__log.debug(f"self.URL : '{self.URL}'")

        try:
            self.device_response = requests.get(self.URL, verify=False, headers=self.headers)
            self.__log.debug(f"deviceResponse.status_code = '{self.device_response.status_code}'")
        except Exception as e:
            print(f"Exception: '{e}.", file=sys.stderr)
            self.__log.critical(f"Fatal error occurred while something.")
            self.__log.exception(f"Exception: '{e}'.")
            return False

        if self.device_response.status_code == 200:
            return json.dumps(self.device_response.json(), indent=4, sort_keys=True)
        else:
            print(f"Unable to retrieve device list from {self.ome_host}", file=sys.stderr)
            self.__log.error(f"Unable to retrieve device list from {self.ome_host}")
            return False

    def get_device_list(self):
        """Verify blah blah blah

    Return codes:
        0 :
        1 :
        2 :
        3 :
        4 :
    """

        ome_list = self._get_device_list()

        dl = pd.read_json(ome_list)

        dl_temp = dl

        while "@odata.nextLink" in dl_temp.keys():
            next_link = dl_temp["@odata.nextLink"][0]
            ome_list = self._get_device_list(next_link=next_link)
            dl_temp = pd.read_json(ome_list)
            dl = pd.concat([dl, dl_temp])

        return dl

    def get_inventory_details(self, device_id):
        """Verify blah blah blah

    Return codes:
        0 :
        1 :
        2 :
        3 :
        4 :
    """

        self.device_id = device_id
        self.__log.debug(f"deviceId : '{self.device_id}'")

        self.inventory_details_url = f"https://{self.ome_host}/api/DeviceService/Devices({self.device_id})/InventoryDetails"
        self.__log.debug(f"url : '{self.inventory_details_url}'")

        self.__log.debug(f"Calling requests.get: {self.inventory_details_url=}")

        try:
            self.device_response = requests.get(self.inventory_details_url,
                                                verify=False,
                                                headers=self.headers)
        except Exception as e:
            print(f"Exception: '{e}'.", file=sys.stderr)
            self.__log.critical(f"Fatal error occurred while something.")
            self.__log.exception(f"Exception: '{e}'.")
            return [self.device_id, False]

        self.__log.debug(f"self.deviceResponse.status_code({self.device_id}) = {self.device_response.status_code}")

        if self.device_response.status_code == 200:
            return [self.device_id, json.dumps(self.device_response.json(), indent=4, sort_keys=True)]
        else:
            print(f"Unable to retrieve device list from {self.ome_host}", file=sys.stderr)
            self.__log.debug(f"self.deviceResponse.status_code({self.device_id}) = {self.device_response.status_code}")
            self.__log.error(f"Unable to retrieve device list from {self.ome_host}", self.ome_host)
            return [self.device_id, False]

    def get_inventory_types(self, device_id):
        """Verify blah blah blah

    Return codes:
        0 :
        1 :
        2 :
        3 :
        4 :
    """

        self.device_id = device_id
        self.__log.debug(f"deviceId : '{self.device_id}'")

        self.inventory_types_url = f"https://{self.ome_host}/api/DeviceService/Devices({self.device_id})/InventoryTypes"
        self.__log.debug(f"url : '{self.inventory_types_url}'")

        try:
            self.device_response = requests.get(self.inventory_types_url,
                                                verify=False,
                                                headers=self.headers)
            self.__log.debug(f"deviceResponse.status_code = '{self.device_response.status_code}'")
        except Exception as e:
            print(f"Exception: '{e}'.", file=sys.stderr)
            self.__log.critical(f"Fatal error occurred while something.")
            self.__log.exception(f"Exception: '{e}'.")
            return False

        if self.device_response.status_code == 200:
            return json.dumps(self.device_response.json(), indent=4, sort_keys=True)
        else:
            print(f"Unable to retrieve device list from {self.ome_host}", file=sys.stderr)
            self.__log.error(f"Unable to retrieve device list from {self.ome_host}")
            return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.
# See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Wed Feb 1 17:53:44 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECScmdb.git $
#

import sys
# import logging
# import re
from datetime import datetime
from pathlib import Path
import autologging

from autologging import logged, traced
from jinja2 import Template

from ecspylibs.initsetup import InitSetup


@traced
@logged
class InitCheck(InitSetup):
    """Initialize the environment using command line parameters and init file."""

    def __init__(self, arguments: dict, default_arguments: dict, config=None, section=None):
        """Setup the InitCheck environment.  Setup default values then process the init file."""

        super().__init__(arguments, default_arguments, config, section)

        TOD = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")

        replace_tod = {
            "TOD": TOD,
        }

        if "--LOG" in self.arguments:
            self.log_file = Path(self.arguments['--LOG']).resolve(strict=False)
            try:
                self.log_file = self.log_file.resolve(strict=True)
            except FileNotFoundError as e:
                self.log_dir = self.log_file.parent.resolve(strict=False)
                try:
                    self.log_dir.mkdir(mode=0o755, parents=True, exist_ok=True)
                    try:
                        self.log_file.touch(mode=0o644, exist_ok=True)
                    except PermissionError as e:
                        print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                    except Exception as e:
                        print(f"\nException: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                except PermissionError as e:
                    print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                    sys.exit(1)
                except Exception as e:
                    print(f"\nException: {e=}\n", file=sys.stderr)
                    sys.exit(1)
            except Exception as e:
                print(f"\nException: {e=}\n", file=sys.stderr)
                sys.exit(1)

            self.arguments['--LOG'] = self.log_file

        if "--output" in self.arguments:
            self.output_file = self.arguments['--output']
            self.output_file = Template(self.output_file).render(replace_tod)
            self.output_file = Path(self.output_file).resolve(strict=False)
            try:
                self.output_file = self.output_file.resolve(strict=True)
            except FileNotFoundError as e:
                self.output_dir = self.output_file.parent.resolve(strict=False)
                try:
                    self.output_dir.mkdir(mode=0o755, parents=True, exist_ok=True)
                    try:
                        self.output_file.touch(mode=0o644, exist_ok=True)
                    except PermissionError as e:
                        print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                    except Exception as e:
                        print(f"\nException: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                except PermissionError as e:
                    print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                    sys.exit(1)
                except Exception as e:
                    print(f"\nException: {e=}\n", file=sys.stderr)
                    sys.exit(1)
            except Exception as e:
                print(f"\nException: {e=}\n", file=sys.stderr)
                sys.exit(1)

            self.arguments['--output'] = self.output_file

        if "--report" in self.arguments:
            self.report_file = self.arguments['--report']
            self.report_file = Template(self.report_file).render(replace_tod)
            self.report_file = Path(self.report_file).resolve(strict=False)
            try:
                self.report_file = self.report_file.resolve(strict=True)
            except FileNotFoundError as e:
                self.report_dir = self.report_file.parent.resolve(strict=False)
                try:
                    self.report_dir.mkdir(mode=0o755, parents=True, exist_ok=True)
                    try:
                        self.report_file.touch(mode=0o644, exist_ok=True)
                    except PermissionError as e:
                        print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                    except Exception as e:
                        print(f"\nException: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                except PermissionError as e:
                    print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                    sys.exit(1)
                except Exception as e:
                    print(f"\nException: {e=}\n", file=sys.stderr)
                    sys.exit(1)
            except Exception as e:
                print(f"\nException: {e=}\n", file=sys.stderr)
                sys.exit(1)

            self.arguments['--report'] = self.report_file

        if "--pw" in self.arguments:
            self.passwd_file = Path(self.arguments['--pw']).resolve(strict=False)
            try:
                self.passwd_file = self.passwd_file.resolve(strict=True)
            except FileNotFoundError as e:
                self.passwd_dir = self.passwd_file.parent.resolve(strict=False)
                try:
                    self.passwd_dir.mkdir(mode=0o755, parents=True, exist_ok=True)
                    try:
                        self.passwd_file.touch(mode=0o644, exist_ok=True)
                    except PermissionError as e:
                        print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                    except Exception as e:
                        print(f"\nException: {e=}\n", file=sys.stderr)
                        sys.exit(1)
                except PermissionError as e:
                    print(f"\nPermission Error: {e=}\n", file=sys.stderr)
                    sys.exit(1)
                except Exception as e:
                    print(f"\nException: {e=}\n", file=sys.stderr)
                    sys.exit(1)
            except Exception as e:
                print(f"\nException: {e=}\n", file=sys.stderr)
                sys.exit(1)

            self.arguments['--pw'] = self.passwd_file

        self.config_name = self.data['name']

        if 'admin' in self.data:
            self.admin_name = None
            self.admin_email = None
            self.admin_phone = None

            if 'name' in self.data['admin']:
                self.admin_name = self.data['admin']['name']

            if 'email' in self.data['admin']:
                self.admin_email = self.data['admin']['email']

            if 'phone' in self.data['admin']:
                self.admin_phone = self.data['admin']['phone']

        if 'email' in self.data:
            self.email_subject = None
            self.email_from = None
            self.email_to = None
            self.email_cc = None
            self.email_text_with_changes = None
            self.email_text_without_changes = None

            if 'subject' in self.data['email']:
                self.email_subject = self.data['email']['subject']

            name = email = None

            if 'from' in self.data['email']:
                if 'name' in self.data['email']['from']:
                    name = self.data['email']['from']['name']

                if 'email' in self.data['email']['from']:
                    email = self.data['email']['from']['email']

                self.email_from = f"{name} <{email}>"

            name = email = None

            if 'to' in self.data['email']:
                self.email_to = []
                for tmp in self.data['email']['to']:
                    if 'name' in tmp:
                        name = tmp['name']

                    if 'email' in tmp:
                        email = tmp['email']

                    self.email_to.append(f"{name} <{email}>")

            name = email = None

            if 'cc' in self.data['email']:
                self.email_cc = []
                for tmp in self.data['email']['cc']:
                    if 'name' in tmp:
                        name = tmp['name']

                    if 'email' in tmp:
                        email = tmp['email']

                    self.email_cc.append(f"{name} <{email}>")

            if 'text' in self.data['email']:
                if 'with_changes' in self.data['email']['text']:
                    self.email_text_with_changes = self.data['email']['text']['with_changes']

                if 'without_changes' in self.data['email']['text']:
                    self.email_text_without_changes = self.data['email']['text']['without_changes']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Wed Feb 1 17:53:44 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECScmdb.git $
#

# Python Standard libraries.

import sys
import pandas as pd

from autologging import logged, traced


@traced
@logged
class FindDifferences:
    """Update cells."""

    def __init__(self, df1, df2, df_comments):
        """ blah """

        self.df1 = df1
        self.df2 = df2
        self.df_comments = df_comments

        self.colors = {
            "blue": "background-color: blue; font-weight: bold",
            "green": "background-color: green; font-weight: bold",
            "red": "background-color: red; font-weight: bold",
            "yellow": "background-color: yellow; font-weight: bold",
        }

    def find_differences(self, sheet):
        """ blah """

        df1_sheet = self.df1[sheet]
        df2_sheet = self.df2[sheet]

        df_style = pd.DataFrame("", index=df2_sheet.index, columns=df2_sheet.columns)

        columns1 = set(df1_sheet.columns.to_list())
        columns2 = set(df2_sheet.columns.to_list())

        columns_deleted = list(columns1.difference(columns2))
        columns_added = list(columns2.difference(columns1))
        columns_common = list(columns1.intersection(columns2))

        self.__log.debug(f"{columns1=}")
        self.__log.debug(f"{columns2=}")
        self.__log.debug(f"{columns_deleted=}")
        self.__log.debug(f"{columns_added=}")
        self.__log.debug(f"{columns_common=}")

        if not df1_sheet.equals(df2_sheet):
            self.__log.debug("df1_sheet != df2_sheet")
            for column in columns_common:
                self.__log.debug(f"{column=}")
                if not df1_sheet[column].equals(df2_sheet[column]):
                    self.__log.debug(f"df1_sheet[{column}] != df2_sheet[{column}]")
                    rows = set(pd.concat([df1_sheet[column],
                                          df2_sheet[column]]).drop_duplicates(keep=False).index.to_list())
                    self.__log.debug(f"{rows=}")
                    if rows:
                        self.__log.debug("in if.")
                        max_rows_df1_sheet = df1_sheet.shape[0]
                        max_rows_df2_sheet = df2_sheet.shape[0]

                        self.__log.debug(f"{max_rows_df1_sheet=}")
                        self.__log.debug(f"{max_rows_df2_sheet=}")

                        for row in rows:
                            self.__log.debug(f"{row=}")
                            if row < max_rows_df1_sheet and row < max_rows_df2_sheet:
                                self.__log.debug("in if.")
                                value1 = df1_sheet.at[row, column]
                                value2 = df2_sheet.at[row, column]

                                self.__log.debug(f"{value1=}")
                                self.__log.debug(f"{value2=}")

                                self.df_comments = self.df_comments.append(
                                    {
                                        "Summary of changes.": f"Cell [{column}, {row + 2}] in sheet '{sheet}' changed (listed in yellow).",
                                        "See sheet tab for details.": f"{sheet}",
                                    }, ignore_index=True)

                                df2_sheet.at[row, column] = f"\"{value1}\" -> \"{value2}\""
                                df_style.at[row, column] = self.colors["yellow"]
                            else:
                                self.__log.debug("in else.")
                                if row >= max_rows_df1_sheet:
                                    self.__log.debug(f"{row=} >= {max_rows_df1_sheet=}")
                                    self.df_comments = self.df_comments.append(
                                        {
                                            "Summary of changes.": f"Row {row + 2} was added to '{sheet}'.",
                                            "See sheet tab for details.": f"{sheet}",
                                        }, ignore_index=True)

                                if row >= max_rows_df2_sheet:
                                    self.__log.debug(f"{row=} >= {max_rows_df2_sheet=}")
                                    self.df_comments = self.df_comments.append(
                                        {
                                            "Summary of changes.": f"Row {row + 2} was removed to '{sheet}'.",
                                            "See sheet tab for details.": f"{sheet}",
                                        }, ignore_index=True)

                                    self.__log.info(f"Adding row to df2_sheet.")
                                    df2_sheet = df2_sheet.append(df1_sheet.iloc[row])
                                    df_style = df_style.append(self.colors["red"])
                    else:
                        self.__log.debug("in else.")

            if columns_deleted or columns_added:
                self.__log.debug(f"{columns_deleted=} or {columns_added=}")
                if columns_added:
                    self.__log.debug(f"{columns_added=}")
                    if len(columns_added) == 1:
                        self.df_comments = self.df_comments.append(
                            {
                                "Summary of changes.": f"Column {columns_added} was added to sheet '{sheet}' (listed in green).",
                                "See sheet tab for details.": f"{sheet}",
                            }, ignore_index=True)
                    else:
                        self.df_comments = self.df_comments.append(
                            {
                                "Summary of changes.": f"Columns {columns_added} were added to sheet '{sheet}' (listed in green).",
                                "See sheet tab for details.": f"{sheet}",
                            }, ignore_index=True)

                    for column in columns_added:
                        df_style[column] = self.colors["green"]

                if columns_deleted:
                    self.__log.debug(f"{columns_deleted=}")
                    if len(columns_deleted) == 1:
                        self.df_comments = self.df_comments.append(
                            {
                                "Summary of changes.": f"Column {columns_deleted} was deleted from sheet '{sheet}' (listed in red).",
                                "See sheet tab for details.": f"{sheet}",
                            }, ignore_index=True)
                    else:
                        self.df_comments = self.df_comments.append(
                            {
                                "Summary of changes.": f"Columns {columns_deleted} were deleted from sheet '{sheet}' (listed in red).",
                                "See sheet tab for details.": f"{sheet}",
                            }, ignore_index=True)

                    for column in columns_deleted:
                        df2_sheet[column] = df1_sheet[column]
                        df_style[column] = self.colors["red"]

        if sheet == "VMWare":
            column_header = "DnsName"
        else:
            column_header = "InventoryType"

        self.__log.info(f"Resort df2_sheet.")
        df2_sheet = df2_sheet.reindex(sorted(df2_sheet.columns), axis=1)
        cols = df2_sheet.columns.tolist()
        cols.insert(0, cols.pop(cols.index(column_header)))
        df2_sheet = df2_sheet.reindex(columns=cols)

        self.__log.info(f"Resort df_style.")
        df_style = df_style.reindex(sorted(df_style.columns), axis=1)
        cols = df_style.columns.tolist()
        cols.insert(0, cols.pop(cols.index(column_header)))
        df_style = df_style.reindex(columns=cols)

        self.__log.info(f"Return [df_style, df2_sheet, self.dfComments].")
        return [df_style, df2_sheet, self.df_comments]

    def set_style(self, df, style=None):
        """ blah """

        return style



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.
# See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Wed Feb 8 12:20:23 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECScmdb.git $
#

import sys

import pandas as pd
from pandas import json_normalize

from datetime import datetime
from docopt import docopt
from getpass import getpass
from pathlib import Path

import logging
import logging.handlers
import autologging
from autologging import logged, traced

from concurrent.futures import ProcessPoolExecutor

import ecspylibs
from ecspylibs.password import Password
from ecspylibs.reapchildren import ReapChildren
from ecspylibs.parallel import Parallel

import ecscmdb
from ecscmdb.api import API
from ecscmdb.initcheck import InitCheck
from ecscmdb.updatecells import UpdateCells


@traced
@logged
def get_col_widths(df):
    return [int(max([len(str(s)) for s in df[col].values] + [len(col)]) + 2.0) for col in df.columns]


@traced
@logged
def set_col_widths(ws, df):
    for i, width in enumerate(get_col_widths(df)):
        ws.set_column(i, i, width)


@traced
@logged
def reformat_text(text, max_len=100):
    """
    Doc String
    """

    text_out = ""
    for i in text.splitlines():
        line = ""
        for w in i.split(" "):
            if len(f"{line} {w}") > max_len:
                text_out += line + "\n"
                line = w
            elif len(w) == 0:
                text_out += "\n"
            elif len(line) == 0:
                line = w
            else:
                line += f" {w}"
        if len(line) > 0:
            text_out += line + "\n"
    return text_out


@traced
@logged
def main():

    """
Program to download the data from the OpenManage DB and build a spreadsheet.

Some default option values listed below can be overridden within the
configuration file.

Usage:
  %(pgm)s [-v] [-L LEVEL] [--LOG=DIR] [-F] [-c CONFIG] [-s SECTION] [-o OUTPUT] [-p PWFILE] [-D] [-P SIZE]
  %(pgm)s [-vl] [-L LEVEL] [--LOG=DIR] [-c CONFIG] [-s SECTION] [-a ID]... [-d ID]... [-p PWFILE] [-D]
  %(pgm)s (-h | --help | -V | --version)

  There are no required options.

Options:
  -h, --help                     Show this help message and exit.
  -V, --version                  Show version information and exit.
  -F, --full                     Show all data, no filtering.
  -c CONFIG, --config=CONFIG     The configuration file.
                                 Default: "%(--config)s"
  -s SECTION, --section=SECTION  The configuration file version (default
                                 defined within the configuration file).
  -o OUTPUT, --output=OUTPUT     Output file or directory.
                                 Default: "%(--output)s"
  -p PWFILE, --pw=PWFILE         The password file.  This file is used when a
                                 login to a website or webpage is required.
                                 Default: "%(--pw)s"
  -l, --list                     List all of the IDs in the password file and
                                 exit.  If both the --list and --verbose
                                 options are included, list both IDs and
                                 Passwords and exit.
  -a ID, --add=ID                Add (or update) an ID and Password and exit.
                                 Program will prompt for the Password to be
                                 saved to the password file.
  -d ID, --delete=ID             Delete an ID (if it exists) from the
                                 password file and exit.
  -v, --verbose                  Print verbose messages.
  -L LEVEL, --log=LEVEL          Print log messages at log value LEVEL.
                                 Valid levels are: TRACE, DEBUG, INFO, WARNING,
                                 ERROR, and CRITICAL.
                                 Default: %(--log)s
  --LOG=DIR                      Log directory.
                                 Default: "%(--LOG)s"
  -D, --dryrun                   Only print out what would be done.
  -P SIZE, --poolsize=SIZE       Call OpenManage using pools of size SIZE.
                                 Default: set by the OS.
    """

    # Get current TOD.

    TOD = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")

    # Get default values.

    FQ_RUN_DIR = Path.cwd().resolve(strict=True)
    FQ_PGM_FILE = Path(__file__).resolve(strict=True)
    assert len(FQ_PGM_FILE.parts) >= 5
    PGM = Path(FQ_PGM_FILE.stem)
    FQ_BASE_DIR = FQ_PGM_FILE.parents[5]
    PKG = Path(__package__)

    ETC_DIR = Path("etc")
    OUTPUT_DIR = Path("output")
    LOG_DIR = Path("log")

    CONF_FILE = PGM.with_suffix(".yml")
    PASSWD_FILE = PGM.with_suffix(".pw")

    OUTPUT_FILE = Path(f"OpenManage-{PGM}.{TOD}.xlsx")
    LOG_FILE = PGM.with_suffix(".log")

    CONF_LIST = [
        FQ_RUN_DIR / ETC_DIR / PKG / CONF_FILE,
        FQ_RUN_DIR / ETC_DIR / CONF_FILE,
        FQ_RUN_DIR / CONF_FILE,
        FQ_BASE_DIR / ETC_DIR / PKG / CONF_FILE,
        FQ_BASE_DIR / ETC_DIR / CONF_FILE,
        FQ_BASE_DIR / CONF_FILE
    ]

    for tmp in CONF_LIST:
        if tmp.is_file():
            FQ_CONF_FILE = tmp.resolve(strict=True)
            break
    else:
        print(f"Couldn't find Configuration File '{CONF_FILE.name}'.", file=sys.stderr)
        sys.exit(1)

    PASSWD_LIST = [
        FQ_RUN_DIR / ETC_DIR / PKG / PASSWD_FILE,
        FQ_RUN_DIR / ETC_DIR / PASSWD_FILE,
        FQ_RUN_DIR / PASSWD_FILE,
        FQ_BASE_DIR / ETC_DIR / PKG / PASSWD_FILE,
        FQ_BASE_DIR / ETC_DIR / PASSWD_FILE,
        FQ_BASE_DIR / PASSWD_FILE
    ]

    for tmp in PASSWD_LIST:
        if tmp.is_file():
            FQ_PASSWD_FILE = tmp.resolve(strict=False)
            break
    else:
        print(f"Couldn't find Password File '{PASSWD_FILE.name}'.", file=sys.stderr)
        sys.exit(1)

    FQ_OUTPUT_DIR = (FQ_RUN_DIR / OUTPUT_DIR).resolve(strict=False)
    FQ_OUTPUT_FILE = (FQ_OUTPUT_DIR / OUTPUT_FILE).resolve(strict=False)

    FQ_LOG_DIR = (FQ_RUN_DIR / LOG_DIR).resolve(strict=False)
    FQ_LOG_FILE = (FQ_LOG_DIR / LOG_FILE).resolve(strict=False)

    help_arguments = {
        "pgm": PGM,
        "--config": FQ_CONF_FILE,
        "--LOG": FQ_LOG_FILE,
        "--log": "WARNING",
        "--output": FQ_OUTPUT_FILE,
        "--pw": FQ_PASSWD_FILE,
    }

    help_text = main.__doc__ % help_arguments

    pgm_version = f"{PGM}: {__package__}({ecscmdb.__version__}), ecspylibs({ecspylibs.__version__})"

    arguments = docopt(help_text, version=pgm_version)

    CONFIG = InitCheck(arguments, help_arguments, config="--config", section="--section")

    arguments = CONFIG.arguments
    data = CONFIG.data

    dry_run = arguments["--dryrun"]
    full = arguments["--full"]
    id_add = arguments["--add"]
    id_delete = arguments["--delete"]
    id_list = arguments["--list"]
    log_fd = arguments["--LOG"]
    log_level = arguments["--log"].upper()
    output = arguments["--output"]
    passwd_file = arguments["--pw"]
    pool_size = arguments["--poolsize"]
    verbose = arguments["--verbose"]
    version = arguments["--version"]

    ome_user = data["OME_Login"]

    output_file = CONFIG.output_file
    log_file = CONFIG.log_file

    # Setup logging.

    log_levels = {
        "CRITICAL": logging.CRITICAL,
        "DEBUG": logging.DEBUG,
        "ERROR": logging.ERROR,
        "INFO": logging.INFO,
        "TRACE": autologging.TRACE,
        "WARNING": logging.WARNING,
    }

    logger = logging.getLogger()
    logger.setLevel(log_levels[log_level])
    log_file = log_fd

    fh = logging.handlers.RotatingFileHandler(log_file, maxBytes=(50 * 1024 ** 2), backupCount=5)

    log_fmt = ""
    log_fmt += "%(asctime)s"
    log_fmt += " %(levelname)-7s"
    log_fmt += " [%(process)5d]"
    log_fmt += " %(module)s.py(%(funcName)s,%(lineno)d):"
    log_fmt += "\t%(message)s"

    formatter = logging.Formatter(log_fmt)

    fh.setFormatter(formatter)

    logger.addHandler(fh)

    logger.info(f"Starting program {pgm_version}.")
    if arguments["--verbose"]:
        print(f"\nStarting program {pgm_version}.\n", file=sys.stdout)

    if dry_run:
        print("\nList of command line arguments.\n", file=sys.stdout)

    for key in sorted(arguments.keys()):
        logger.debug(f"arguments['{key}']\t= '{arguments[key]}'")
        if dry_run:
            print(f"arguments['{key}']\t= '{arguments[key]}'", file=sys.stdout)

    del key

    my_locals = locals()
    my_locals_keys = my_locals.keys()
    my_locals_vals = my_locals.values()
    del my_locals

    if dry_run:
        print("\nList of all variables.\n", file=sys.stdout)

    for k, v in zip(my_locals_keys, my_locals_vals):
        logger.debug(f"{k:20} = '{v}'")
        if dry_run:
            print(f"{k:20} = '{v}'", file=sys.stdout)

    if dry_run:
        print("", file=sys.stdout)
        sys.exit(0)

    if id_list or id_add or id_delete:
        logger.info("ID Maintenance.")
        passwd_data = Password(passwd_file)

        if id_add:
            logger.info("Add/Update ID.")
            for id in id_add:
                logger.debug(f"Add/Update ID {id}.")
                pw = getpass(prompt=f"Enter password for user '{id}': ")
                passwd_data.set(id, pw)
            del pw

        if id_delete:
            logger.info("Delete ID.")
            for id in id_delete:
                logger.debug(f"Delete ID {id}")
                passwd_data.delete(id)

        if id_list:
            logger.info("List IDs.")
            if verbose:
                passwd_data.listPWs()
            else:
                passwd_data.listIDs()

        del passwd_data
        logger.info(f"Ending program {pgm_version}.")
        if verbose:
            print(f"\nEnding program {pgm_version}.\n", file=sys.stdout)
        sys.exit(0)

    # The real stuff goes here.

    logger.info("Initialize the Open Manage REST API.")
    ome_api = API(ome_user, passwd_file)

    logger.info(f"login into the Open Manage REST API with with user '{ome_user}'.")
    ome_api.login()

    logger.info("Get list of all Open Manage devices.")
    df_device_list = ome_api.get_device_list()

    logger.debug(f"df_device_list = '{df_device_list}'.")

    logger.info("Initialize Pandas.")
    df_spreadsheet = pd.DataFrame()
    logger.info("Initialize ExcelWriter.")
    excel = pd.ExcelWriter(output)

    records_vmware = {}
    records_inv_details = {}
    device_list = []
    device_id = {}

    logger.info("Create list of VMware devices.")
    if verbose:
        print("Create list VMware devices.", file=sys.stdout)

    for device in df_device_list["value"]:
        for device_mgt in device["DeviceManagement"]:
            for device_tag in ["DnsName", "MacAddress"]:
                if device_tag not in records_vmware:
                    records_vmware[device_tag] = []
                records_vmware[device_tag].append(device_mgt[device_tag])

        for device_tag in ["DeviceName", "DeviceServiceTag", "Model", "InventoryDetails@odata.navigationLink"]:
            if device_tag not in records_vmware:
                records_vmware[device_tag] = []
            records_vmware[device_tag].append(device[device_tag])

        device_name = device["DeviceName"][:31]
        id = device["Id"]
        model = device["Model"]

        device_list.append(device_name)
        device_id[device_name] = id

    logger.info("Create the VMWare worksheet.")
    if verbose:
        print("Create the VMWare worksheet.", file=sys.stdout)

    logger.info("Create df_spreadsheet from pd.DataFrame(records_vmware).")
    df_spreadsheet = df_spreadsheet.append(pd.DataFrame(records_vmware))

    logger.info("Sort df_spreadsheet.")
    df_spreadsheet = df_spreadsheet.sort_values(by=df_spreadsheet.columns.tolist())

    logger.info("Reindex df_spreadsheet.")
    df_spreadsheet = df_spreadsheet.reindex(sorted(df_spreadsheet.columns), axis=1)

    logger.info("Move column DnsName to column 1.")
    cols = df_spreadsheet.columns.tolist()
    cols.insert(0, cols.pop(cols.index("DnsName")))
    df_spreadsheet = df_spreadsheet.reindex(columns=cols)

    logger.info("Write df_spreadsheet to excel file as VMWare.")
    df_spreadsheet.to_excel(excel, "VMWare", index=False)

    logger.info("Update column widths of worksheet VMWare.")
    worksheet = excel.sheets["VMWare"]
    set_col_widths(worksheet, df_spreadsheet)

    device_list.sort()

    if full:
        logger.info("All Rows/Columns will be displayed.")

    else:
        logger.info("Configure which Rows and Columns to copy for each device type.")

        update_cells = UpdateCells()

        fields = []

        fields.append(
            [
                "serverProcessors",
                [
                    "InstanceId",
                    "InventoryType",
                    "ModelName",
                    "NumberOfCores",
                ]
            ]
        )

        fields.append(
            [
                "serverFcCards",
                [
                    "DeviceDescription",
                    "DeviceName",
                    "InventoryType",
                    "VirtualWwn",
                    "VirtualWwpn",
                    "Wwn",
                    "Wwpn",
                ]
            ]
        )

        fields.append(
            [
                "serverOperatingSystems",
                [
                    "Hostname",
                    "InventoryType",
                    "OsName",
                ]
            ]
        )

        fields.append(
            [
                "serverArrayDisks",
                [
                    "BusType",
                    "DiskNumber",
                    "InventoryType",
                    "MediaType",
                    "ModelNumber",
                    "SerialNumber",
                    "Size",
                ]
            ]
        )

        fields.append(
            [
                "serverMemoryDevices",
                [
                    "DeviceDescription",
                    "InstanceId",
                    "InventoryType",
                    "ManufacturerDate",
                    "Name",
                    "SerialNumber",
                    "Size",
                    "Speed",
                    "TypeDetails",
                ]
            ]
        )

        fields.append(
            [
                "serverNetworkInterfaces",
                [
                    "InventoryType",
                    "NicId",
                    [
                        "Ports",
                        update_cells.update_ports,
                    ]
                ]
            ]
        )

        fields.append(
            [
                "chassisSlotsList",
                [
                    "DnsName",
                    "InventoryType",
                    "Model",
                    "Name",
                    "Number",
                    "Occupied",
                    "ServiceTag",
                ]
            ]
        )

        fields.append(
            [
                "chassisControllerList",
                [
                    "InventoryType",
                    "Name",
                ]
            ]
        )

        fields.append(
            [
                "deviceManagement",
                [
                    "DnsName",
                    [
                        "EndPointAgents",
                        update_cells.update_end_point_agents,
                    ],
                    "InstrumentationName",
                    "InventoryType",
                    "IpAddress",
                    "MacAddress",
                    "ManagementId",
                    "ManagementType.Description",
                    "ManagementType.Name",
                ]
            ]
        )

        fields.append(
            [
                "nicInformation",
                [
                    "Description",
                    "DeviceId",
                    "InventoryType",
                    "Ipv4_address",
                    "Mac_address",
                    "Subnet_mask",
                ]
            ]
        )

        rename_columns = {
            "Ports": "Mac Address",
        }

    sort_field_names = [
        "InventoryType",
        "DeviceDescription",
        "DeviceName",
        "InstanceId",
        "NicId",
        "CapabilityType.CapabilityId",
        "CapabilityType.Description",
        "CapabilityType.IdOwner",
        "CapabilityType.Name",
        "DiskNumber",
        "DnsName",
        "EndDate",
        "EntitlementId",
        "Id",
        "IdOwner",
        "LicenseDescription",
        "LicenseType.LicenseId",
        "LicenseType.Name",
        "Name",
        "Number",
        "SasAddress",
        "SerialNumber",
        "ServiceTag",
        "SlotNumber",
        "SoldDate",
    ]

    logger.info("Create a worksheet for each device.")
    if verbose:
        print("Create a worksheet for each device.", file=sys.stdout)

    device_count = 0
    device_length = len(device_list)

    logger.debug(f"Calling Parallel(ome_api.getInventoryDetails, process_pool=True, pool_size={pool_size})")
    if verbose:
        if pool_size:
            print(f"Creating {pool_size} pools to process {device_length} calls to the Open Manage REST API.", file=sys.stdout)
        else:
            print(f"Creating one pool per CPU to process {device_length} calls to the Open Manage REST API.", file=sys.stdout)

    try:
        get_inv_details = Parallel(ome_api.get_inventory_details, ProcessPoolExecutor, pool_size=pool_size)
    except Exception as e:
        logger.critical("Error processing ome_api.getInventoryDetails.  Process terminating.")
        logger.exception(f"Exception: {e=}")
        print("Error processing ome_api.getInventoryDetails.  Process terminating.", file=sys.stderr)
        sys.exit(1)

    logger.debug(f"{pool_size} pools created.")
    if verbose:
        if pool_size:
            print(f"{pool_size} pools created.", file=sys.stdout)
        else:
            print(f"One pool per CPU created.", file=sys.stdout)

    device_ids = list(device_id.values())
    logger.debug(f"{device_ids=}")

    logger.debug(f"Calling get_inv_details.run({device_ids})")
    if verbose:
        print(f"Calling OpenManage with {pool_size} pools to retrieve Device details.", file=sys.stdout)

    try:
        results = get_inv_details.run(device_ids)
    except Exception as e:
        logger.critical(f"Error processing get_inv_details.run({device_ids}).  Process terminating.")
        logger.exception(f"Exception: {e=}")
        print(f"Error processing get_inv_details.run({device_ids}).  Process terminating.", file=sys.stderr)
        sys.exit(1)

    if verbose:
        print(
            f"The call to OpenManage took {get_inv_details.run_time:.2f} seconds",
            f"to fetch {device_length} VMware Device records.",
            file=sys.stdout
        )
    logger.debug(f"get_inv_details.run took {get_inv_details.run_time} seconds to fetch {device_length} Devices.")

    for result in results:
        id, data = result[0], result[1]
        logger.debug(f"{id=}")
        logger.debug(f"{type(data)=}")
        if data:
            logger.debug(f"Found data for id '{id}'.")
            records_inv_details[id] = data
        else:
            logger.error(f"Did not find data for id '{id}'.")
            print(f"Error processing get_inv_details.run({device_ids}).  Process terminating.", file=sys.stderr)
            sys.exit(1)

    for device in device_list:
        device_count += 1
        id = device_id[device]

        logger.info(f"Creating worksheet[{device_count}:{device_length}] for device '{device}[{id}]'.")
        if verbose:
            print(f"Creating worksheet[{device_count}:{device_length}] for device '{device}[{id}]'.", file=sys.stdout)
        if id in records_inv_details:
            logger.debug(f"records_inv_details[{id}] : {records_inv_details[id]}")
        else:
            logger.error(f"Couldn't find id '{id}' in records_inv_details.")
            print("Error processing Detail records.  Process terminating.", file=sys.stderr)
            sys.exit(1)

        df_inv_details = pd.read_json(records_inv_details[id])
        logger.debug(f"df_inv_details : {df_inv_details}")
        logger.debug(f"df_inv_details['value'] : {df_inv_details['value']}")
        sheet = json_normalize(df_inv_details["value"], "InventoryInfo", ["InventoryType"])

        if not full:
            logger.info(f"If device '{device}' is a server or enclosure, filter out unnecessary data.")

            out_fields = {}

            for row, columns in fields:
                logger.info(f"Looking for row '{row}' for device '{device}'.`")

                if sheet["InventoryType"].str.contains(row).any():
                    logger.info(f"Found row '{row}' for device '{device}'.`")

                    if row not in out_fields:
                        out_fields[row] = []

                    for column in columns:
                        if type(column) is list:
                            column, cmd = column
                        else:
                            cmd = None

                        logger.info(f"Looking for row:column '{row}:{column}' for device '{device}'.")

                        if column in sheet.columns:
                            logger.info(f"Found row:column '{row}:{column}' for device '{device}'.`")
                            out_fields[row].append(column)
                            logger.debug(f"out_fields[{row}] : {out_fields[row]}")

                        else:
                            logger.info(f"Can't find row:column '{row}:{column}' for device '{device}'.`")

                else:
                    logger.info(f"Can't find row '{row}' for device '{device}'.")

            out = {}

            for key in out_fields.keys():
                logger.debug(f"out_fields[{key}] : '{out_fields[key]}'")
                out[key] = sheet.loc[(sheet["InventoryType"] == key), out_fields[key]]

            logger.info(f"Recreate sheet for device '{device}'.")
            sheet = pd.concat([out[key] for key in out], sort=False)

            logger.debug(f"Sheet = '{sheet}'.")

            for row, columns in fields:
                logger.info(f"Looking for row '{row}' for device '{device}'.`")

                if sheet["InventoryType"].str.contains(row).any():
                    logger.info(f"Found row '{row}' for device '{device}'.`")

                    for column in columns:
                        if type(column) is list:
                            column, cmd = column
                        else:
                            continue

                        logger.info(f"Looking for row:column '{row}:{column}' for device '{device}'.")

                        if column in sheet.columns:
                            logger.info(f"Found row:column '{row}:{column}' for device '{device}'.`")
                            cmd(sheet)

                else:
                    logger.info(f"Can't find row '{row}' for device '{device}'.")

            logger.info(f"Rename, if necessary, column names for device '{device}'.")
            sheet = sheet.rename(columns=rename_columns)

        logger.info(f"Sort the columns for sheet '{device}'.")
        sheet = sheet.reindex(sorted(sheet.columns), axis=1)

        logger.info(f"Move column 'InventoryType' to the beginning of the spreadsheet for device '{device}'.")
        cols = sheet.columns.tolist()
        logger.debug(f"sheet : {sheet}")
        logger.debug(f"sheet.columns : {sheet.columns}")
        logger.debug(f"sheet.columns.tolist() : {sheet.columns.tolist()}")
        cols.insert(0, cols.pop(cols.index("InventoryType")))
        sheet = sheet.reindex(columns=cols)

        sort_fields = []
        logger.debug(f"sort_field_names = {sort_field_names}")

        for field_name in sort_field_names:
            logger.debug(f"field_name = {field_name}")
            logger.debug(f"Is '{field_name}' in {sheet.columns.tolist()}")
            if field_name in sheet.columns:
                if field_name not in sort_fields:
                    logger.debug(f"Add '{field_name}' to {sort_fields}")
                    sort_fields.append(field_name)
                else:
                    logger.debug(f"'{field_name}' already in {sort_fields}")

        if sort_fields:
            logger.debug(f"'{device}.sort_fields' = {sort_fields}")
            sheet.sort_values(by=sort_fields, inplace=True, ignore_index=True)

        logger.info(f"Update the spreadsheet for device {device}.")
        sheet.to_excel(excel, device, index=False)

        worksheet = excel.sheets[device]
        set_col_widths(worksheet, sheet)

    logger.info("Save and close the spreadsheet file.")
    if verbose:
        print("Save and close the spreadsheet file.", file=sys.stdout)
    excel.save()
    excel.close()

    rc = ReapChildren()
    try:
        rc.reapChildren()
    except Exception as e:
        logger.critical("Fatal error occurred in reapChildren().")
        logger.exception(f"Exception: '{e}'.")

    logger.info(f"Ending program {pgm_version}.")
    if verbose:
        print(f"\nEnding program {pgm_version}.\n", file=sys.stdout)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Tue Jan 31 14:48:53 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECScmdb.git $
#

# Python Standard libraries.

from autologging import logged, traced
from pandas import json_normalize


@traced
@logged
class UpdateCells:
    """Update cells."""

    def __init__(self):
        self.end_point_agents = None
        self.cols = None
        self.macs = None
        self.partitions = None
        return

    def update_ports(self, sheet):
        for self.i in sheet.loc[sheet["InventoryType"].str.contains("serverNetworkInterfaces")].index:
            self.partitions = json_normalize(sheet.loc[self.i, "Ports"], "Partitions")
            self.__log.debug(f"Partitions : {self.partitions}")

            for self.debugName in self.partitions:
                self.__log.debug(f"Partition Name : {self.debugName}")

            if "CurrentMacAddress" in self.partitions:
                self.macs = []
                for self.mac in self.partitions.CurrentMacAddress:
                    self.__log.debug(f"Mac Address : {self.mac}")
                    self.macs.append(self.mac)
                sheet[f"Ports.CurrentMacAddress"] = ""
                sheet.at[self.i, "Ports.CurrentMacAddress"] = self.macs

        self.cols = sheet.columns.tolist()
        self.__log.debug(f"Columns before delete Ports: {self.cols}")

        self.__log.info(f"Delete column 'Ports'.")
        sheet.drop("Ports", axis=1, inplace=True)

        self.cols = sheet.columns.tolist()
        self.__log.debug(f"Columns after delete Ports: {self.cols}")

    def update_end_point_agents(self, sheet):
        for self.i in sheet.loc[sheet["InventoryType"].str.contains("deviceManagement")].index:
            self.end_point_agents = json_normalize(sheet.loc[self.i, "EndPointAgents"])
            self.__log.debug(f"endPointAgents : {self.end_point_agents}")
            self.__log.debug(f"endPointAgents.columns : {self.end_point_agents.columns}")

            for self.column in self.end_point_agents.columns:
                self.__log.info(f"endPointAgents.column : {self.column}")
                self.__log.debug(f"endPointAgents.{self.column} : {self.end_point_agents[self.column]}")
                self.__log.debug(f"endPointAgents.{self.column}.values : {self.end_point_agents[self.column].values}")
                self.__log.debug(f"Type(endPointAgents.{self.column}.values) : {type(self.end_point_agents[self.column].values)}")

                self.__log.info(f"Try to sort endPointAgents.{self.column}.values.")
                self.__log.debug(f"endPointAgents.{self.column}.values before {self.end_point_agents[self.column].values}")
                try:
                    self.end_point_agents[self.column].values.sort()
                    self.__log.debug(f"endPointAgents.{self.column}.values after {self.end_point_agents[self.column].values}")
                except Exception as e:
                    self.__log.warning(f"Couldn't sort 'endPointAgents.{self.column}.values'.")
                    self.__log.exception(f"Exception: '{e}'.")

                sheet[f"EndPointAgents.{self.column}"] = ""
                sheet.at[self.i, f"EndPointAgents.{self.column}"] = self.end_point_agents[self.column].values

        self.cols = sheet.columns.tolist()
        self.__log.debug(f"Columns before delete EndPointAgents: {self.cols}")

        self.__log.info(f"Delete column 'EndPointAgents'.")
        sheet.drop("EndPointAgents", axis=1, inplace=True)

        self.cols = sheet.columns.tolist()
        self.__log.debug(f"Columns after delete EndPointAgents: {self.cols}")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ecscmdb']

package_data = \
{'': ['*']}

install_requires = \
['ecspylibs==v1.1.15']

entry_points = \
{'console_scripts': ['cmdb = ecscmdb.cmdb:main',
                     'cmdbdiff = ecscmdb.cmdbdiff:main']}

setup_kwargs = {
    'name': 'ecscmdb',
    'version': '1.6.22',
    'description': 'Dump the OpenManage database.',
    'long_description': '# ECScmdb\n    Author:            $Id: Thomas R. Stevenson <aa0026@wayne.edu> $\n    Last Changed:      $Date: Wed Feb 8 18:51:00 2023 -0500 $\n    URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECScmdb.git $\n\n# Summary\n\n    cmdb:     Download the devices in the OpenManage database into a\n              spreadsheet, one work sheet for each device.\n\n    cmdbdiff: Compare two cmdb spreadsheets for differences.\n\n# Installation\n\n Use Python virtual environment and install package ecscmdb.\n\n# Version\n\n## cmdb --version\n    cmdb: ecscmdb(1.6.20), ecspylibs(1.1.14)\n\n## cmdbdiff --version\n    cmdbdiff: ecscmdb(1.6.20), ecspylibs(1.1.14)\n\n# Help\n\n## cmdb --help\n\n    Program to download the data from the OpenManage DB and build a spreadsheet.\n\n    Some default option values listed below can be overridden within the\n    configuration file.\n\n    Usage:\n      cmdb [-v] [-L LEVEL] [--LOG=DIR] [-F] [-c CONFIG] [-s SECTION] [-o OUTPUT] [-p PWFILE] [-D] [-P SIZE]\n      cmdb [-vl] [-L LEVEL] [--LOG=DIR] [-c CONFIG] [-s SECTION] [-a ID]... [-d ID]... [-p PWFILE] [-D]\n      cmdb (-h | --help | -V | --version)\n\n      There are no required options.\n\n    Options:\n      -h, --help                     Show this help message and exit.\n      -V, --version                  Show version information and exit.\n      -F, --full                     Show all data, no filtering.\n      -c CONFIG, --config=CONFIG     The configuration file.\n                                     Default: "/home/tom/Run/ECScmdb/Testing/etc/ecscmdb/cmdb.yml"\n      -s SECTION, --section=SECTION  The configuration file version (default\n                                     defined within the configuration file).\n      -o OUTPUT, --output=OUTPUT     Output file or directory.\n                                     Default: "/home/tom/Run/ECScmdb/Testing/output/OpenManage-cmdb.2023-02-08-18-14-30.xlsx"\n      -p PWFILE, --pw=PWFILE         The password file.  This file is used when a\n                                     login to a website or webpage is required.\n                                     Default: "/home/tom/Run/ECScmdb/Testing/etc/ecscmdb/cmdb.pw"\n      -l, --list                     List all of the IDs in the password file and\n                                     exit.  If both the --list and --verbose\n                                     options are included, list both IDs and\n                                     Passwords and exit.\n      -a ID, --add=ID                Add (or update) an ID and Password and exit.\n                                     Program will prompt for the Password to be\n                                     saved to the password file.\n      -d ID, --delete=ID             Delete an ID (if it exists) from the\n                                     password file and exit.\n      -v, --verbose                  Print verbose messages.\n      -L LEVEL, --log=LEVEL          Print log messages at log value LEVEL.\n                                     Valid levels are: TRACE, DEBUG, INFO, WARNING,\n                                     ERROR, and CRITICAL.\n                                     Default: WARNING\n      --LOG=DIR                      Log directory.\n                                     Default: "/home/tom/Run/ECScmdb/Testing/log/cmdb.log"\n      -D, --dryrun                   Only print out what would be done.\n      -P SIZE, --poolsize=SIZE       Call OpenManage using pools of size SIZE.\n                                     Default: set by the OS.\n\n## cmdbdiff --help\n\n    Program to analyze two spreadsheets for differences.\n\n    Some default option values listed below can be overridden within the initialization file.\n\n    Usage:\n      cmdbdiff [-v] [-L LEVEL] [--LOG=DIR] [-c CONFIG] [-s SECTION] [-r REPORT] [-D] SPREADSHEET1 SPREADSHEET2\n      cmdbdiff (-h | --help | -V | --version)\n\n      Variables SPREADSHEET1 and SPREADSHEET2 are required, all other parameters are optional.\n\n    Options:\n      -h, --help                          Show this help message and exit.\n      -V, --version                       Show version information and exit.\n      -c CONFIG, --config=CONFIG          The configuration file.\n                                          Default: "/home/tom/Run/ECScmdb/Testing/etc/ecscmdb/cmdbdiff.yml"\n      -s SECTION, --section=SECTION       The configuration file version (default\n                                          defined within the configuration file).\n      -r REPORT, --report=REPORT          Report directory or file.\n      -v, --verbose                       Print verbose messages.\n      -L LEVEL, --log=LEVEL               Print log messages at log value LEVEL.\n                                          Valid levels are: TRACE, DEBUG, INFO, WARNING,\n                                          ERROR, and CRITICAL.\n                                          Default: "WARNING"\n      --LOG=DIR                           Log Directory,\n                                          Default: "/home/tom/Run/ECScmdb/Testing/log/cmdbdiff.log"\n      -D, --dryrun                        Only print out what would be done.\n \n# Git information\n    The GIT Home can be found [Here][CMDB].\n    The README file can be found [Here][README].\n    The LICENSE file can be found [Here][LICENSE].\n    The ChangeLog file can be found [Here][CHANGELOG].\n    The pyproject.toml file can be found [Here][PYPROJECT].\n\n# About me\n    My contact information can be found [Here][About Me].\n\n[CMDB]: https://git.wayne.edu/ECS_Projects/ECScmdb\n[README]: https://git.wayne.edu/ECS_Projects/ECScmdb/-/blob/master/README.md\n[LICENSE]: https://git.wayne.edu/ECS_Projects/ECScmdb/-/blob/master/LICENSE.txt\n[CHANGELOG]: https://git.wayne.edu/ECS_Projects/ECScmdb/-/blob/master/ChangeLog\n[PYPROJECT]: https://git.wayne.edu/ECS_Projects/ECScmdb/-/blob/master/pyproject.toml\n[About Me]: https://About.Me/Thomas.R.Stevenson\n',
    'author': 'Thomas R. Stevenson',
    'author_email': 'aa0026@wayne.edu',
    'maintainer': 'Thomas R. Stevenson',
    'maintainer_email': 'aa0026@wayne.edu',
    'url': 'https://git.wayne.edu/ECS_Projects/ECScmdb.git',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
